/* ===========================================================================
 * $RCSfile: PkCl.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/01 14:14:13 $
 * $Revision: 1.3 $
 */
package COM.rl.obf;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import COM.rl.util.*;
import COM.rl.obf.classfile.*;

/**
 * Base to package and class tree item.
 *
 * @author      Mark Welsh
 */
abstract public class PkCl extends TreeItem
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    /** Owns a list of classes. */
    protected Hashtable cls = new Hashtable();


    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    /** Ctor. */
    public PkCl(TreeItem parent, String name)
    {
        super(parent, name);
    }

    /** Get a class by name. */
    public Cl getClass(String name) throws Exception {return (Cl)cls.get(name);}

    /** Get an Enumeration of classes directly beneath this PkCl. */
    public Enumeration getClassEnum() {return cls.elements();}

    /** Get an Enumeration of all classes (outer and inner) in the tree beneath this PkCl. */
    public Enumeration getAllClassEnum() 
    {
        Vector allClasses = new Vector();
        addAllClasses(allClasses);
        return allClasses.elements();
    }
    private void addAllClasses(Vector allClasses)
    {
        // List classes and recursively compose a list of all inner classes
        for (Enumeration enum = cls.elements(); enum.hasMoreElements(); )
        {
            Cl cl = (Cl)enum.nextElement();
            allClasses.addElement(cl);
            cl.addAllClasses(allClasses);
        }
    }

    /** Return number of classes. */
    public int getClassCount() {return cls.size();}

    /** Add a class to the list of owned classes. */
    abstract public Cl addClass(String name, String superName, String[] interfaceNames) throws Exception;

    /** Add a class to the list of owned classes. */
    public Cl addClass(boolean isInnerClass, String name, String superName, String[] interfaceNames) throws Exception
    {
        Cl cl = getClass(name);

        // Remove placeholder if present
        PlaceholderCl plClassItem = null;
        if (cl instanceof PlaceholderCl)
        {
            plClassItem = (PlaceholderCl)cl;
            cls.remove(name);
            cl = null;
        }

        // Add the class, if not already present
        if (cl == null)
        {
            cl = new Cl(this, isInnerClass, name, superName, interfaceNames);
            cls.put(name, cl);
        }

        // Copy over the inner class data from the placeholder, if any
        if (plClassItem != null)
        {
            for (Enumeration enum = plClassItem.getClassEnum(); enum.hasMoreElements(); )
            {
                Cl innerCl = (Cl)enum.nextElement();
                innerCl.setParent(cl);
                cl.addClass(innerCl);
            }
        }
        return cl;
    }

    /** Add a placeholder class to our list of owned classes, to be replaced later by the full class. */
    abstract public Cl addPlaceholderClass(String name) throws Exception;

    /** Add a placeholder class to our list of owned classes, to be replaced later by the full class. */
    public Cl addPlaceholderClass(boolean isInnerClass, String name) throws Exception
    {
        Cl cl = getClass(name);
        if (cl == null)
        {
            cl = new PlaceholderCl(this, isInnerClass, name);
            cls.put(name, cl);
        }
        return cl;
    }

    /** Generate unique obfuscated names for this namespace. */
    public void generateNames() throws Exception
    {
        generateNames(cls);
    }

    /** Generate unique obfuscated names for a given namespace. */
    protected void generateNames(Hashtable hash) throws Exception
    {
        Vector vec = new Vector();
        for (Enumeration enum = hash.elements(); enum.hasMoreElements(); )
        {
            TreeItem ti = (TreeItem)enum.nextElement();
            if (ti.isFixed())
            {
                vec.addElement(ti.getOutName());
            }
        }
        String[] noObfNames = new String[vec.size()];
        for (int i = 0; i < noObfNames.length; i++)
        {
            noObfNames[i] = (String)vec.elementAt(i);
        }
        NameMaker nm = new KeywordNameMaker(noObfNames, false, true);
        for (Enumeration enum = hash.elements(); enum.hasMoreElements(); )
        {
            TreeItem ti = (TreeItem)enum.nextElement();
            if (!ti.isFixed())
            {
                ti.setOutName(nm.nextName(null));
            }
        }
    }
}

