/* ===========================================================================
 * $RCSfile: TreeItem.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/01 14:14:13 $
 * $Revision: 1.2 $
 */
package COM.rl.obf;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import COM.rl.util.*;
import COM.rl.obf.classfile.*;

/**
 * Item that forms a tree structure and can represent a package level, a class,
 * or a method or field.
 *
 * @author      Mark Welsh
 */
public class TreeItem
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    protected boolean isSynthetic;  // Is a method or field Synthetic?
    protected int access;  // Access level (interpret using java.lang.reflect.Modifier)
    protected ClassTree classTree = null;   // Our owner
    protected TreeItem parent = null;       // Our immediate parent
    protected String sep = ClassFile.SEP_REGULAR; // Separator preceeding this level's name
    private String inName = null;         // Original name of this item
    private String outName = null;        // Output name of this item
    private boolean isFixed = false; // Has the name been fixed in some way?
    private boolean isFromScript = false; // Is this script constrained?
    private boolean isFromScriptMap = false; // Is this script_map constrained?


    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    /** Ctor. */
    public TreeItem(TreeItem parent, String name)
    {
        this.parent = parent;
        this.inName = name;
        if (parent != null)
        {
            classTree = parent.classTree;
        }
    }

    /** Return the modifiers. */
    public int getModifiers() {return access;}

    /** Return the original name of the entry. */
    public String getInName() {return inName;}

    /** Set the output name of the entry. */
    public void setOutName(String outName) 
    {
        // DEBUG
        //if (isFixed) 
        //{
        //    System.out.println("BIG TROUBLE: " + inName + " -> " + this.outName + " -> " + outName);
        //}
        this.outName = outName;
        isFixed = true;
    }

    /** Return the output name of the entry, obfuscated or original. */
    public String getOutName() {return outName != null ? outName : inName;}

    /** Signal that this constraint came from a user script line. */
    public void setFromScript() {isFromScript = true;}

    /** Signal that this constraint came from a map script line. */
    public void setFromScriptMap() {isFromScriptMap = true;}

    /** Has the entry been fixed already? */
    public boolean isFixed() {return isFixed;}

    /** Is this constrained by a user script line? */
    public boolean isFromScript() {return isFromScript;}

    /** Is this constrained by a map script line? */
    public boolean isFromScriptMap() {return isFromScriptMap;}

    /** Is a method or field Synthetic? */
    public boolean isSynthetic() {return isSynthetic;}

    /** Set the parent in the tree -- used when stitching in a Cl to replace a PlaceholderCl. */
    public void setParent(TreeItem parent) {this.parent = parent;}

    /** Get the parent in the tree. */
    public TreeItem getParent() {return parent;}

    /** Construct and return the full original name of the entry. */
    public String getFullInName()
    {
        if (parent == null)
        {
            return "";
        }
        else if (parent.parent == null)
        {
            return getInName();
        }
        else
        {
            return parent.getFullInName() + sep + getInName();
        }
    }

    /** Construct and return the full obfuscated name of the entry. */
    public String getFullOutName()
    {
        if (parent == null)
        {
            return "";
        }
        else if (parent.parent == null)
        {
            return getOutName();
        }
        else
        {
            return parent.getFullOutName() + sep + getOutName();
        }
    }
}
