/* ===========================================================================
 * $RCSfile: Utf8CpInfo.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/07/01 15:24:33 $
 * $Revision: 1.1.1.1 $
 */
package COM.rl.obf.classfile;

import java.io.*;
import java.util.*;

/**
 * Representation of a 'UTF8' entry in the ConstantPool.
 *
 * @author      Mark Welsh
 */
public class Utf8CpInfo extends CpInfo
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    private int u2length;
    private byte[] bytes;
    private String utf8string;


    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    protected Utf8CpInfo()
    {
        super(CONSTANT_Utf8);
    }

    /** Ctor used when appending fresh Utf8 entries to the constant pool. */
    public Utf8CpInfo(String s) throws Exception
    {
        super(CONSTANT_Utf8);
        setString(s);
        refCount = 1;
    }

    /** Decrement the reference count, blanking the entry if no more references. */
    public void decRefCount() throws Exception
    {
        super.decRefCount();
        if (refCount == 0)
        {
            clearString();
        }
    }

    /** Return UTF8 data as a String. */
    public String getString() throws Exception
    {
        if (utf8string == null)
        {
            utf8string = new String(bytes, "UTF8");
        }
        return utf8string;
    }

    /** Set UTF8 data as String. */
    public void setString(String str) throws Exception
    {
        utf8string = str;
        bytes = str.getBytes("UTF8");
        u2length = bytes.length;
    }

    /** Set the UTF8 data to empty. */
    public void clearString() throws Exception
    {
        u2length = 0;
        bytes = new byte[0];
        utf8string = null;
        getString();
    }

    /** Read the 'info' data following the u1tag byte. */
    protected void readInfo(DataInput din) throws Exception
    {
        u2length = din.readUnsignedShort();
        bytes = new byte[u2length];
        din.readFully(bytes);
        getString();
    }

    /** Write the 'info' data following the u1tag byte. */
    protected void writeInfo(DataOutput dout) throws Exception
    {
        dout.writeShort(u2length);
        dout.write(bytes);
    }
}
