/* ===========================================================================
 * $RCSfile: PkClEntry.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/27 02:36:07 $
 * $Revision: 1.2 $
 */
package COM.rl.obf.gui;

import java.io.*;
import java.util.*;
import COM.rl.util.*;
import COM.rl.obf.*;

/**
 * An entry in the tree control list.
 *
 * @author      Mark Welsh
 */
public class PkClEntry implements TreeEntry
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    private PkCl pkcl;
    private boolean isOpen;
    private String string;
    private int depth;


    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    /** Ctor. */
    public PkClEntry(int depth, PkCl pkcl)
    {
        this.depth = depth;
        this.pkcl = pkcl;
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < depth; i++)
        {
            sb.append("        ");
        }
        if (pkcl instanceof Pk)
        {
            string = sb.toString() + "[Package]" + pkcl.getInName();
        }
        else
        {
            string = sb.toString() + pkcl.getInName();
        }
    }

    /** Return the wrapped PkCl. */
    public PkCl getPkCl() {return pkcl;}

    /** Can the element be opened. */
    public boolean canOpen() {return true;}

    /** Is the element open in the tree? */
    public boolean isOpen() {return isOpen;}

    /** Set if the element is open in the tree. */
    public void setOpen(boolean isOpen) {this.isOpen = isOpen;}

    /** String representation of the element for the List. */
    public String toString() {return string;}

    /** Number of children. */
    public int childCount()
    {
        int totalCount = pkcl.getClassCount();
        if (pkcl instanceof Pk)
        {
            totalCount += ((Pk)pkcl).getPackageCount();
        }
        return totalCount;
    }

    /** Return the children TreeEntry's. */
    public Enumeration elements() {
        Vector vec = new Vector();
        try {
            // For a package, list sub-packages
            if (pkcl instanceof Pk) {
                PkClEntry[] sortPackages = new PkClEntry[((Pk)pkcl).getPackageCount()];
                int i = 0;
                for (Enumeration enum = ((Pk)pkcl).getPackageEnum(); enum.hasMoreElements(); i++) {
                    sortPackages[i] = new PkClEntry(depth + 1, (PkCl)enum.nextElement());
                }
                Sort.quicksort(sortPackages, 
                               new Compare() {
                    public boolean isLess(Object o1, Object o2) {
                        return ((PkClEntry)o1).getPkCl().getInName().compareTo(((PkClEntry)o2).getPkCl().getInName()) < 0;
                    }
                });
                for (i = 0; i < sortPackages.length; i++) {
                    vec.addElement(sortPackages[i]);
                }
            }
            // Now list class entries
            PkClEntry[] sortClasses = new PkClEntry[pkcl.getClassCount()];
            int i = 0;
            for (Enumeration enum = pkcl.getClassEnum(); enum.hasMoreElements(); i++) {
                sortClasses[i] = new PkClEntry(depth + 1, (PkCl)enum.nextElement());
            }
            Sort.quicksort(sortClasses, 
                           new Compare() {
                public boolean isLess(Object o1, Object o2) {
                    return ((PkClEntry)o1).getPkCl().getInName().compareTo(((PkClEntry)o2).getPkCl().getInName()) < 0;
                }
            });
            for (i = 0; i < sortClasses.length; i++) {
                vec.addElement(sortClasses[i]);
            }
        } catch (Exception e) {
            vec = new Vector();
        }
        return vec.elements();
    }
}
