/* ===========================================================================
 * $RCSfile: Header.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/27 02:36:08 $
 * $Revision: 1.2 $
 */
package COM.rl.util.rfc822;

import java.io.*;
import java.util.*;

/**
 * An RFC822 'header' is a 'tag' / 'value' pair.
 *
 * @author Mark Welsh
 */
public class Header
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    private String tag;
    private String value;

    // Class Methods ---------------------------------------------------------
    /** Parse a header from the specified String. */
    public static Header parse(String line)
    {
        Header header = null;
        if (line != null) 
        {
            int pos = line.indexOf(':');
            if (pos != -1)
            {
                header = new Header(line.substring(0, pos).trim(), line.substring(pos + 1).trim());
            }
        }
        return header;
    }

    // Instance Methods ------------------------------------------------------
    /** Ctor. */
    public Header(String tag, String value)
    {
        this.tag = (tag == null ? "" : tag);
        this.value = (value == null ? "" : value);
    }

    /** Return the tag. */
    public String getTag() {return tag;}

    /** Return the value. */
    public String getValue() {return value;}

    /** Test equality of headers. */
    public boolean equals(Object o)
    {
        if (o instanceof Header)
        {
            Header header = (Header)o;
            if (header.getTag().equals(getTag()) && header.getValue().equals(getValue()))
            {
                return true;
            }
        }
        return false;
    }

    /** Print String rep of this object to a java.io.Writer. */
    public void writeString(Writer writer) throws IOException
    {
        writer.write(getTag() + ": " + getValue());
    }

    /** Return String rep of this object. */
    public String toString()
    {
        return getTag() + ": " + getValue();
    }
}
