/* ===========================================================================
 * $RCSfile: Section.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/27 02:36:09 $
 * $Revision: 1.2 $
 */
package COM.rl.util.rfc822;

import java.io.*;
import java.util.*;

/**
 * An RFC822 section is a list of 'header's (tag/value pairs).
 *
 * @author Mark Welsh
 */
public class Section
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    private Vector headers;


    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    /** Blank section. */
    public Section()
    {
        headers = new Vector();
    }

    /** Append a header to this section. */
    public void add(Header header)
    {
        headers.addElement(header);
    }

    /** Append a header to this section. */
    public void add(String tag, String value)
    {
        add(new Header(tag, value));
    }

    /** Remove the header from this section. */
    public void remove(Header header)
    {
        headers.removeElement(header);
    }

    /** Return an Enumeration of headers. */
    public Enumeration elements()
    {
        return headers.elements();
    }

    /** Does the section contain a header matching the specified one? */
    public boolean hasHeader(Header queryHeader)
    {
        if (queryHeader != null)
        {
            for (Enumeration enum = headers.elements(); enum.hasMoreElements(); )
            {
                Header header = (Header)enum.nextElement();
                if (queryHeader.equals(header)) 
                {
                    return true;
                }
            }
        }
        return false;
    }

    /** Find a header matching the specified tag, or null if none. */
    public Header findTag(String tag)
    {
        // Check params
        if (tag == null) return null;
        
        // For now, do linear search of headers
        for (Enumeration enum = headers.elements(); enum.hasMoreElements(); )
        {
            Header header = (Header)enum.nextElement();
            if (tag.equals(header.getTag())) 
            {
                // Found
                return header;
            }
        }

        // Not found
        return null;
    }

    /** Find a header matching the specified value, or null if none. */
    public Header findValue(String value)
    {
        // Check params
        if (value == null) return null;
        
        // For now, do linear search of headers
        for (Enumeration enum = headers.elements(); enum.hasMoreElements(); )
        {
            Header header = (Header)enum.nextElement();
            if (value.equals(header.getValue())) 
            {
                // Found
                return header;
            }
        }

        // Not found
        return null;
    }

    /** Print String rep of this object to a java.io.Writer. */
    public void writeString(Writer writer) throws IOException
    {
        for (Enumeration enum = headers.elements(); enum.hasMoreElements(); )
        {
            ((Header)enum.nextElement()).writeString(writer);
            writer.write("\015\012");
        }
        writer.write("\015\012");
    }

    /** Return String rep of this object. */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        for (Enumeration enum = headers.elements(); enum.hasMoreElements(); )
        {
            sb.append(((Header)enum.nextElement()).toString());
            sb.append("\015\012");
        }
        sb.append("\015\012");
        return sb.toString();
    }
}
