/* ===========================================================================
 * $RCSfile: SectionList.java,v $
 * ===========================================================================
 *
 * RetroGuard -- an obfuscation package for Java classfiles.
 *
 * Copyright (c) 1999 Mark Welsh (markw@retrologic.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author may be contacted at markw@retrologic.com 
 *
 *
 * $Date: 1998/10/27 02:36:10 $
 * $Revision: 1.2 $
 */
package COM.rl.util.rfc822;

import java.io.*;
import java.util.*;

/**
 * A list of RFC822 sections, usually representing a file.
 *
 * @author Mark Welsh
 */
public class SectionList
{
    // Constants -------------------------------------------------------------


    // Fields ----------------------------------------------------------------
    private Vector sections;

    // Class Methods ---------------------------------------------------------


    // Instance Methods ------------------------------------------------------
    /** Construct with no initial sections. */
    public SectionList()
    {
	sections = new Vector();
    }

    /** Parse the stream, appending the sections found there to our list. */
    public void parse(InputStream in)
    {
	// Wrap the stream for text reading
	BufferedReader reader = new BufferedReader(new InputStreamReader(in));

	// Read until end of file
	String line;
	Section section = null;
        try
        {
            while ((line = reader.readLine()) != null)
            {
                // If at end of section, close section
                if (section != null && line.indexOf(':') == -1)
                {
                    add(section);
                    section = null;
                }
                
                // If at start of section, skip non-header lines
                if (section == null && line.indexOf(':') == -1)
                {
                    continue;
                }
                
                // Start or continue section
                if (section == null)
                {
                    section = new Section();
                }
                section.add(Header.parse(line));
            }
        }
        catch (IOException e)
        {
            // Unexpected EOF during readLine() can cause this - just terminate the read quietly
            return;
        }
    }

    /** Add a Section to the list. */
    public void add(Section section)
    {
        sections.addElement(section);
    }

    /** Return an Enumeration of sections. */
    public Enumeration elements()
    {
        return sections.elements();
    }

    /** Find the first section in the list containing the matching header. */
    public Section find(String tag, String value)
    {
        return find(new Header(tag, value));
    }

    /** Find the first section in the list containing the matching header. */
    public Section find(Header header)
    {
        for (Enumeration enum = elements(); enum.hasMoreElements(); )
        {
            Section section = (Section)enum.nextElement();
            if (section.hasHeader(header))
            {
                return section;
            }
        }
	return null;
    }

    /** Print String rep of this object to a java.io.Writer. */
    public void writeString(Writer writer) throws IOException
    {
        for (Enumeration enum = elements(); enum.hasMoreElements(); )
        {
            ((Section)enum.nextElement()).writeString(writer);
        }
    }

    /** Return String rep of this object. */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        for (Enumeration enum = elements(); enum.hasMoreElements(); )
        {
            sb.append(((Section)enum.nextElement()).toString());
        }
        return sb.toString();
    }
}
